@@ -17,6 +17,7 @@ class App(object):
     Application class
     """
     url_pattern = dict()
+    debug = False
 
     def __init__(self, app_path=None):
         if app_path is None:
@@ -26,6 +27,7 @@ class App(object):
         webkit_web_view = WebKit.WebView()
         settings = webkit_web_view.get_settings()
         settings.set_property('enable-universal-access-from-file-uris', True)
+        settings.set_property('enable-file-access-from-file-uris', True)
         settings.set_property('default-encoding', 'utf-8')
         window.set_default_size(800, 600)
         scrollWindow = Gtk.ScrolledWindow()
@@ -43,7 +45,7 @@ class App(object):
             self.on_web_view_resource_response_received)
         webkit_web_view.connect(
             'resource-load-finished',
-            self.on_resource_load_finished)
+            self.on_web_view_resource_load_finished)
         webkit_web_view.connect(
             'navigation_policy_decision_requested',
             self.on_navigation_policy_decision_requested)
@@ -55,6 +57,13 @@ class App(object):
         webkit_main_frame.connect(
             'resource-response-received',
             self.on_web_frame_resource_response_received)
+        webkit_main_frame.connect(
+            'resource-load-finished',
+            self.on_web_frame_resource_load_finished)
+        webkit_main_frame.connect(
+            'resource-load-failed',
+            self.on_web_frame_resource_load_failed)
+
         window.show_all()
         self.window = window
         self.webkit_web_view = webkit_web_view
@@ -70,11 +79,12 @@ class App(object):
 
         if len(match_list) > 1:
             raise Exception('Found more than one matched urls')
+            return None
 
         try:
             m = match_list[0]
         except:
-            raise Exception('Can\'t find matched url')
+            return None
 
         args = list(m.groups())
         kw = m.groupdict()
@@ -96,7 +106,8 @@ class App(object):
         """
         status = webkitView.get_load_status()
         if status == status.FINISHED:
-            print 'Load finished'
+            if self.debug is True:
+                print 'Load finished'
 
     def on_navigation_policy_decision_requested(
             self,
@@ -105,7 +116,8 @@ class App(object):
             webkit_network_request,
             webkit_web_navigation_action,
             webkit_web_policy_dicision):
-        print 'navigation_policy_decision_requested'
+        if self.debug is True:
+            print 'navigation_policy_decision_requested'
 
     def on_web_view_resource_request_starting(
             self,
@@ -114,7 +126,8 @@ class App(object):
             web_resource,
             network_request,
             network_response=None):
-        print 'web_view_resource_request_starting'
+        if self.debug is True:
+            print 'web_view_resource_request_starting'
 
     def on_web_view_resource_response_received(
             self,
@@ -123,7 +136,15 @@ class App(object):
             web_resource,
             network_response,
             *arg, **kw):
-        print 'web_view Resource response received'
+        if self.debug is True:
+            print 'web_view_resource_response_received'
+
+    def on_web_view_resource_load_finished(
+            self,
+            web_view, web_frame, web_resource,
+            *args, **kw):
+        if self.debug is True:
+            print 'web_view_resource_load_finished'
 
     def on_web_frame_resource_request_starting(
             self,
@@ -131,12 +152,15 @@ class App(object):
             web_resource,
             network_request,
             network_response=None):
-        print 'web_frame_resource_request_starting'
+        if self.debug is True:
+            print 'web_frame_resource_request_starting'
         url = urlparse.unquote(network_request.get_uri())
         url = urlparse.urlparse(url.decode('utf-8'))
-        if url.scheme == 'app':
-            if url.netloc == '':
-                result = self._url_map_to_function(url.path)
+        if url.netloc == '':
+            # Try mapping request path to function. `return`.
+            # If there's no mapped function then serve it as static file.
+            result = self._url_map_to_function(url.path)
+            if result:
                 # Make sure result is <tuple>
                 if isinstance(result, unicode) or \
                         isinstance(result, str):
@@ -148,9 +172,16 @@ class App(object):
                 f.write(content)
                 f.close()
                 network_request.set_uri('file://' + tmp_file_path + '?tmp=1')
-            elif url.netloc == 'file':
-                file_path = self.app_path + url.path
+            else:
+                splitted_path = url.path.split('/')
+                if splitted_path[1] == 'tmp':
+                    splitted_path.pop(1)
+                url_path = os.path.join(*splitted_path)
+                file_path = os.path.join(self.app_path, url_path)
                 file_path = os.path.normcase(file_path)
+                file_path = os.path.normpath(file_path)
+                if not(os.path.exists(file_path)):
+                    raise Exception('Not found: ' + file_path)
                 network_request.set_uri('file://' + file_path)
 
     def on_web_frame_resource_response_received(
@@ -159,30 +190,47 @@ class App(object):
             web_resource,
             network_response,
             *arg, **kw):
-        print 'web_frame Resource response received'
+        if self.debug is True:
+            print 'web_frame_resource_response_received'
         url = urlparse.urlparse(network_response.get_uri())
         url = urlparse.urlparse(url.path)
         query = urlparse.parse_qs(url.query)
         if 'tmp' in query:
-            print url.path
             os.remove(url.path)
 
-    def on_resource_load_finished(
+    def on_web_frame_resource_load_finished(
             self,
-            web_view, web_frame, web_resource,
-            *args, **kw):
-        print 'resource load finished'
+            web_frame,
+            web_resource,
+            *arg, **kw):
+        if self.debug is True:
+            print 'on_web_frame_resource_load_finished'
+
+    def on_web_frame_resource_load_failed(
+            self,
+            web_frame,
+            web_resource,
+            *arg, **kw):
+        if self.debug is True:
+            print 'on_web_frame_resource_load_failed'
 
     def run(self):
-        self.webkit_web_view.load_uri('app:///')
+        index = self._url_map_to_function('/')
+        if self.debug is True:
+            print self.app_path
+        self.webkit_web_view.load_string(
+            index,
+            mime_type='text/html',
+            encoding='utf-8',
+            base_uri='/',
+        )
         Gtk.main()
 
-    def _get_app_path(self, path='.'):
-        print self.__file__
-
-    def request_handler():
-        pass
-
 
 def response(content=None, mimetype='text/html'):
+    """Make response tuple
+
+    Potential features to be added
+      - Parameters validation
+    """
     return (content, mimetype)
